from rest_framework.authentication import BaseAuthentication
from rest_framework.exceptions import NotAuthenticated

from django.contrib.auth.models import AnonymousUser
from django.conf import settings

from bcmr.models import *


class HeaderAuthentication(BaseAuthentication):

    def __init__(self, realm="API"):
        self.realm = realm

    def authenticate(self, request, **kwargs):
        try:
            if request.method == 'GET':
                request.user = AnonymousUser()
                return (request.user, None)

            bcmr_auth_token = request.META.get(settings.AUTH_HEADER)
            if not bcmr_auth_token:
                if request.method == 'POST':
                    request.user = AnonymousUser()
                    return (request.user, None)
                else:
                    raise NotAuthenticated(detail='BCMR authorization token required', code=403)

            auth_token = AuthToken.objects.get(id=bcmr_auth_token)
            if auth_token:
                model = request.path_info.split('/')[2]
                pk = request.path_info.split('/')[3]
                
                if pk:
                    model_obj = None
                    try:
                        if model == 'registries':
                            model_obj = Registry.objects.get(id=pk)
                        elif model == 'tokens':
                            model_obj = Token.objects.get(category=pk)
                    except:
                        pass
                        
                    if model_obj:
                        if str(model_obj.owner.id) != bcmr_auth_token:
                            raise NotAuthenticated(
                                detail='Unauthorized action: only owner can modify this data',
                                code=403
                            )

                request.user = AnonymousUser()
                return (request.user, None)
        except AuthToken.DoesNotExist as dne:
            raise NotAuthenticated(detail='Invalid auth token', code=403)
        except KeyError as ke:
            raise NotAuthenticated(detail='Client Not Found', code=403)



from django_filters import rest_framework as filters

from bcmr.models import *


class RegistryFilter(filters.FilterSet):
    class Meta:
        model = Registry
        fields = (
            'owner',
        )


class TokenFilter(filters.FilterSet):
    class Meta:
        model = Token
        fields = (
            'owner',
        )



# Generated by Django 3.1.6 on 2023-02-13 02:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bcmr', '0002_authtoken'),
    ]

    operations = [
        migrations.AddField(
            model_name='registry',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='registries', to='bcmr.authtoken'),
        ),
        migrations.AddField(
            model_name='token',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='tokens', to='bcmr.authtoken'),
        ),
    ]



from django import forms

from bcmr.models import Token


class TokenForm(forms.ModelForm):
    class Meta:
        model = Token
        fields = (
            'category',
            'name',
            'description',
            'symbol',
            'decimals',
            'icon',
        )



# Generated by Django 3.1.6 on 2023-02-14 04:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bcmr', '0003_auto_20230213_0237'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='authtoken',
            options={'verbose_name_plural': 'Auth Tokens'},
        ),
        migrations.AlterModelOptions(
            name='registry',
            options={'ordering': ('name',), 'verbose_name_plural': 'Registries'},
        ),
    ]
